@@ -1,12 +1,15 @@
 
 /*
- *  $Id: funcDef.c,v 1.59 2004/02/02 03:31:50 bkorb Exp $
+ *  $Id: funcDef.c,v 4.17 2006/11/27 01:52:22 bkorb Exp $
+ *
+ *  Time-stamp:        "2006-11-26 15:55:10 bkorb"
+ *  Last Committed:    $Date: 2006/11/27 01:52:22 $
  *
  *  This module implements the DEFINE text function.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2006 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -21,8 +24,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 typedef int (tCmpProc)( const void*, const void* );
@@ -36,13 +39,26 @@ struct def_list {
 tSCC zNoResolution[] = "Could not resolve macro name: ``%s''";
 tSCC zTplInvoked[] = "Template macro %s invoked with %d args\n";
 
-STATIC tDefList* linkTwins( tDefList* pDL, tDefList* pNext, int* pCt );
+static tDefList* linkTwins( tDefList* pDL, tDefList* pNext, int* pCt );
 
-STATIC int   orderDefList( const void* p1, const void* p2 );
-STATIC void  prepInvokeArgs( tMacro* pMac );
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static int
+orderDefList( const void* p1, const void* p2 );
 
+static tDefList*
+linkTwins( tDefList* pDL, tDefList* pNext, int* pCt );
 
-STATIC int
+static void
+prepInvokeArgs( tMacro* pMac );
+
+static void
+build_defs( int defCt, tDefList* pList );
+/* = = = END-STATIC-FORWARD = = = */
+
+tMacro* mLoad_Debug( tTemplate* pT, tMacro* pMac, tCC** ppzScan );
+
+static int
 orderDefList( const void* p1, const void* p2 )
 {
     tDefEntry* pDL1 = (tDefEntry*)p1;
@@ -60,7 +76,7 @@ orderDefList( const void* p1, const void* p2 )
 }
 
 
-STATIC tDefList*
+static tDefList*
 linkTwins( tDefList* pDL, tDefList* pNext, int* pCt )
 {
     tDefList* pN;
@@ -103,11 +119,11 @@ linkTwins( tDefList* pDL, tDefList* pNext, int* pCt )
  *  for name-value assignments that are only to live for the duration
  *  of the processing of the user defined macro.
  */
-EXPORT void
+LOCAL void
 parseMacroArgs( tTemplate* pT, tMacro* pMac )
 {
     char*        pzScan = pT->pzTemplText + pMac->ozText;
-    int          ct     = 0;
+    u_int        ct     = 0;
     tDefList*    pDL;
     tDefList*    pN;
 
@@ -125,8 +141,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
     while (*pzScan != NUL) {
         ct++;
         if (! isalpha( *pzScan )) {
-            fprintf( stderr, "On macro argument # %d:\n%s\n",
-                     ct, pzScan );
+            fprintf( stderr, "On macro argument # %d:\n%s\n", ct, pzScan );
             AG_ABEND_IN( pT, pMac, "no macro arg name" );
         }
 
@@ -229,8 +244,9 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
              */
             if ((pzScan - pDL->pzExpr) < 24) {
                 char* pz = (char*)AGALOC( 24, "quoted string" );
-                memcpy( (void*)pz, pDL->pzExpr, (pzScan - pDL->pzExpr) );
+                memcpy((void*)pz, pDL->pzExpr, (size_t)(pzScan - pDL->pzExpr));
                 pDL->pzExpr = pz;
+                manageAllocatedData( pz );
             }
             spanQuote( pDL->pzExpr );
             /* FALLTHROUGH */
@@ -269,7 +285,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
          *  if the strings compare equal.
          */
         pDL = (tDefList*)pMac->res;
-        qsort( (void*)pDL, ct, sizeof( tDefList ), orderDefList );
+        qsort( (void*)pDL, (size_t)ct, (size_t)sizeof(tDefList), orderDefList );
 
         /*
          *  Now, link them all together.  Singly linked list.
@@ -287,7 +303,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
              *  THEN it is a "twin".  Link twins on the twin list.
              */
             if (streqvcmp( pDL->de.pzDefName, pN->de.pzDefName ) == 0) {
-                pN = linkTwins( pDL, pN, &ct );
+                pN = linkTwins( pDL, pN, (int*)&ct );
                 if (ct <= 0)
                     break;  /* pN is now invalid */
             }
@@ -299,7 +315,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
 }
 
 
-STATIC void
+static void
 prepInvokeArgs( tMacro* pMac )
 {
     char*  pzText;
@@ -385,7 +401,7 @@ mFunc_Debug( tTemplate* pT, tMacro* pMac )
  *
  *  Build up a definition context created by passed-in macro arguments
  */
-STATIC void
+static void
 build_defs( int defCt, tDefList* pList )
 {
     tDefEntry* pDefs = &(pList->de);
@@ -428,15 +444,16 @@ build_defs( int defCt, tDefList* pList )
                 fprintf( pfTrace, "Scheme eval for arg %d:\n\t`%s'\n",
                          pCurMacro->sibIndex - defCt, pList->pzExpr );
             }
-            res = ag_eval( pList->pzExpr );
 
-            if (gh_string_p( res )) {
+            res = ag_eval( pList->pzExpr );
+            if (AG_SCM_STRING_P( res )) {
                 AGDUPSTR( pList->de.val.pzText, ag_scm2zchars(res, "eval res"),
                           "dup eval res" );
             }
-            else if (gh_number_p( res )) {
+            else if (AG_SCM_NUM_P( res )) {
                 pList->de.val.pzText = AGALOC( 16, "number buf" );
-                snprintf( pList->de.val.pzText, 16, "%ld", gh_scm2ulong( res ));
+                snprintf(pList->de.val.pzText, (size_t)16, "%ld",
+                         gh_scm2ulong( res ));
             }
             else
                 AGDUPSTR( pList->de.val.pzText, zNil, "empty string" );
@@ -460,6 +477,8 @@ build_defs( int defCt, tDefList* pList )
  *  what:    Define a user AutoGen macro
  *  cindex:  define macro
  *  handler_proc:
+ *  load_proc:
+ *  unload-proc:
  *
  *  desc:
  *
@@ -476,11 +495,11 @@ build_defs( int defCt, tDefList* pList )
  *  @end example
  *
  *  Once the macro has been defined, this new macro can be invoked by
- *  specifying the macro name as the first token after the start macro
- *  marker.  Alternatively, you may make the invocation explicitly invoke a
- *  defined macro by specifying @code{INVOKE} in the macro invocation.  If
- *  you do that, the macro name can be computed with an expression that gets
- *  evaluated every time the INVOKE macro is encountered.  @xref{INVOKE}.
+ *  specifying the macro name as the first token after the start macro marker.
+ *  Alternatively, you may make the invocation explicitly invoke a defined
+ *  macro by specifying @code{INVOKE} (@pxref{INVOKE}) in the macro
+ *  invocation.  If you do that, the macro name can be computed with an
+ *  expression that gets evaluated every time the INVOKE macro is encountered.
  *
  *  Any remaining text in the macro invocation will be used to create new
  *  name/value pairs that only persist for the duration of the processing of
@@ -502,10 +521,25 @@ build_defs( int defCt, tDefList* pList )
  *  @item
  *  Sequences of strings are not concatenated.
  *  @end enumerate
+ *
+ *  @quotation
+ *  @strong{NB:} The macro is extracted from the template as the template is
+ *  scanned.  You cannot conditionally define a macro by enclosing it in an
+ *  @code{IF}/@code{ENDIF} (@pxref{IF}) macro pair.  If you need to dynamically
+ *  select the format of a @code{DEFINE}d macro, then put the flavors into
+ *  separate template files that simply define macros.  @code{INCLUDE}
+ *  (@pxref{INCLUDE}) the appropriate template when you have computed which
+ *  you need.
+ *  @end quotation
+ *
+ *  Due to this, it is acceptable and even a good idea to place all the
+ *  @code{DEFINE} macros at the end of the template.  That puts the main
+ *  body of the template at the beginning of the file.
 =*/
 /*=macfunc ENDDEF
  *
  *  what:   Ends a macro definition.
+ *  in-context:
  *
  *  desc:
  *    This macro ends the @code{DEFINE} function template block.
@@ -528,7 +562,7 @@ mFunc_Define( tTemplate* pT, tMacro* pMac )
     if (OPT_VALUE_TRACE > TRACE_NOTHING) {
         fprintf( pfTrace, zTplInvoked, pT->pzTplName, defCt );
         if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
-            fprintf( pfTrace, zFileLine, pCurTemplate->pzFileName,
+            fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                      pMac->lineNo );
     }
 
@@ -567,6 +601,15 @@ mFunc_Define( tTemplate* pT, tMacro* pMac )
 }
 
 
+void
+mUnload_Define( tMacro* pMac )
+{
+    void* p = (void*)(pMac->res);
+    if (p != NULL)
+        AGFREE(p);
+}
+
+
 /*=macfunc INVOKE
  *
  *  handler_proc:
@@ -632,9 +675,10 @@ mFunc_Invoke( tTemplate* pT, tMacro* pMac )
      */
     macName = eval( pT->pzTemplText + pMac->ozName );
 
-    pInv = findTemplate( ag_scm2zchars( macName, "macro name" ));
+    pzText = ag_scm2zchars( macName, "macro name" );
+    pInv = findTemplate( pzText );
     if (pInv == NULL) {
-        pzText = aprf( zNoResolution, ag_scm2zchars( macName, "macro name" ));
+        pzText = aprf( zNoResolution, pzText );
         AG_ABEND_IN( pT, pMac, pzText );
         /* NOTREACHED */
     }
@@ -678,21 +722,21 @@ mLoad_Define( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
      *  THEN set up the "DEFINE" block callout table.
      *  It is the standard table, except entries are inserted
      *  for functions that are enabled only while processing
-     *  a DEFINE block (i.e. "ENDDEF")
+     *  a DEFINE block (viz. "ENDDEF" and removing "DEFINE").
      */
     if (apDefineLoad[0] == NULL) {
         memcpy( (void*)apDefineLoad, apLoadProc, sizeof( apLoadProc ));
         apDefineLoad[ FTYP_ENDDEF ] = &mLoad_Ending;
+        apDefineLoad[ FTYP_DEFINE ] = &mLoad_Bogus;
     }
     papLoadProc = apDefineLoad;
 
     {
-        const char*    pzScan = *ppzScan;  /* text after macro */
-        const char*    pzSrc  = (const char*)pMac->ozText; /* macro text */
+        char const*    pzScan = *ppzScan;  /* text after macro */
+        char const*    pzSrc  = (char const*)pMac->ozText; /* macro text */
         int            macCt  = pT->macroCt - (pMac - pT->aMacros);
-        int            fnameSize = strlen( pT->pzFileName ) + 1;
 
-        size_t alocSize = sizeof( *pNewT ) + fnameSize
+        size_t alocSize = sizeof( *pNewT )
             + (macCt * sizeof( tMacro )) + strlen( pzScan ) + 0x100;
         alocSize &= ~0x0F;
 
@@ -701,19 +745,22 @@ mLoad_Define( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
          */
         pNewT = (tTemplate*)AGALOC( alocSize, "AG macro definition" );
         memset( (void*)pNewT, 0, alocSize );
-        pNewT->magic    = pT->magic;
-        pNewT->fd       = 0;
-        pNewT->descSize = alocSize;
-        pNewT->macroCt  = macCt;
-        pNewT->pzFileName = (char*)(pNewT->aMacros + macCt);
-        memcpy( (void*)pNewT->pzFileName, pT->pzFileName, fnameSize );
-        pzCopy = pNewT->pzTplName = (void*)(pNewT->pzFileName + fnameSize);
+        pNewT->magic      = pT->magic;
+        pNewT->descSize   = alocSize;
+        pNewT->macroCt    = macCt;
+        pNewT->pzTplFile  = strdup(pT->pzTplFile);
+
+        pzCopy = pNewT->pzTplName = (void*)(pNewT->aMacros + macCt);
         if (! isalpha( *pzSrc ))
             AG_ABEND_IN( pT, pMac, zNameNeeded );
 
         while (ISNAMECHAR(*pzSrc))  *(pzCopy++) = *(pzSrc++);
     }
 
+    if (OPT_VALUE_TRACE >= TRACE_BLOCK_MACROS)
+        fprintf( pfTrace, "Defining macro %s from %s\n",
+                 pNewT->pzTplName, pNewT->pzTplFile );
+
     *(pzCopy++) = NUL;
     pNewT->pzTemplText = pzCopy;
     pNewT->pNext = pzCopy+1;
@@ -723,75 +770,74 @@ mLoad_Define( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 
     {
         tMacro* pMacEnd = parseTemplate( pNewT->aMacros, ppzScan );
+        int     ct;
 
         /*
          *  Make sure all of the input string was *NOT* scanned.
          */
         if (*ppzScan == NULL)
-            AG_ABEND_IN( pT, pMac, "parse ended unexpectedly" );
+            AG_ABEND_IN( pNewT, pNewT->aMacros, "parse ended unexpectedly" );
 
-        pNewT->macroCt = pMacEnd - &(pNewT->aMacros[0]);
+        ct = pMacEnd - pNewT->aMacros;
 
         /*
          *  IF there are empty macro slots,
          *  THEN pack the text
          */
-        if ((void*)pMacEnd < (void*)(pNewT->pzFileName)) {
-            int  delta = pNewT->pzFileName - (char*)pMacEnd;
-            int  size  = (pNewT->pNext - pNewT->pzFileName);
-            memcpy( (void*)pMacEnd, (void*)(pNewT->pzFileName),
-                    size );
-            pNewT->pzFileName  -= delta;
+        if (ct < pNewT->macroCt) {
+            int   delta = sizeof(tMacro) * (pNewT->macroCt - ct);
+            void* data  = (pNewT->pzTplName == NULL) ?
+                pNewT->pzTemplText : pNewT->pzTplName;
+            size_t size = pNewT->pNext - (char*)data;
+            memmove( (void*)pMacEnd, data, size );
+
             pNewT->pzTemplText -= delta;
             pNewT->pNext       -= delta;
-            if (pNewT->pzTplName != 0)
-                pNewT->pzTplName -= delta;
+            pNewT->pzTplName   -= delta;
+            pNewT->macroCt      = ct;
         }
     }
 
     /*
-     *  Adjust the sizes.  Remove absolute pointers.
-     *  Reallocate to the correct size.  Restore
-     *  the offsets to pointer values.
+     *  Adjust the sizes.  Remove absolute pointers.  Reallocate to the correct
+     *  size.  Restore the offsets to pointer values.
      */
-    pNewT->descSize     = pNewT->pNext - (char*)pNewT;
-    pNewT->descSize    += 0x40;
-    pNewT->descSize    &= ~0x3F;
-    pNewT->pzFileName  -= (long)pNewT;
-    pNewT->pzTplName   -= (long)pNewT;
-    pNewT->pzTemplText -= (long)pNewT;
-    pNewT = (tTemplate*)AGREALOC( (void*)pNewT, pNewT->descSize,
-                                  "resize AG macro definition" );
-    pNewT->pzFileName  += (long)pNewT;
-    pNewT->pzTplName   += (long)pNewT;
-    pNewT->pzTemplText += (long)pNewT;
+    {
+        size_t sz = pNewT->pNext - (char*)pNewT;
+        if (sz < pNewT->descSize) {
+            pNewT->descSize     = sz;
+            pNewT->pzTplName   -= (long)pNewT;
+            pNewT->pzTemplText -= (long)pNewT;
+            pNewT = (tTemplate*)AGREALOC( (void*)pNewT, pNewT->descSize,
+                                          "resize AG macro definition" );
+            if (pNewT == NULL)
+                AG_ABEND( "failed to resize AG macro" );
+
+            pNewT->pzTplName   += (long)pNewT;
+            pNewT->pzTemplText += (long)pNewT;
+        }
+    }
 
-    /*
-     *  We cannot reallocate a smaller array because
-     *  the entries are all linked together and
-     *  realloc-ing it may cause it to move.
-     */
 #if defined( DEBUG_ENABLED )
     if (HAVE_OPT( SHOW_DEFS )) {
         tSCC zSum[] = "loaded %d macros from %s\n"
             "\tBinary template size:  0x%X\n\n";
-        fprintf( pfTrace, zSum, pNewT->macroCt, pNewT->pzFileName,
+        fprintf( pfTrace, zSum, pNewT->macroCt, pNewT->pzTplFile,
                  pNewT->descSize );
     }
 #endif
 
     pNewT->pNext  = (char*)pNamedTplList;
     pNamedTplList = pNewT;
-    papLoadProc = papLP;
+    papLoadProc   = papLP;
     memset( (void*)pMac, 0, sizeof(*pMac) );
-    pCurTemplate = pT;
+    pCurTemplate  = pT;
     return pMac;
 }
 /*
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of agen5/funcDef.c */
