@@ -1,12 +1,16 @@
-
 /*
  *  agTempl.c
- *  $Id: tpProcess.c,v 4.6 2006/01/24 23:22:17 bkorb Exp $
+ *  $Id: tpProcess.c,v 4.15 2006/12/15 23:24:27 bkorb Exp $
+ *
  *  Parse and process the template data descriptions
+ *
+ * Time-stamp:        "2006-12-15 14:14:15 bkorb"
+ * Last Committed:    $Date: 2006/12/15 23:24:27 $
+ *
  */
 
 /*
- *  AutoGen copyright 1992-2005 Bruce Korb
+ *  AutoGen copyright 1992-2006 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -92,31 +96,30 @@ static void
 doStdoutTemplate( tTemplate* pTF )
 {
     tSCC zNoSfx[] = "* NONE *";
-    tSCC zBadR[]  = "%sBogus return from setjmp:  %d\n";
-    int  jmpcode  = setjmp( fileAbort );
+    tSCC zBadR[]  = "%sBogus return from setjmp\n";
     SCM  res;
     tCC* pzRes;
 
     pzLastScheme = NULL; /* We cannot be in Scheme processing */
 
-    switch (jmpcode) {
+    switch (setjmp (fileAbort)) {
     case SUCCESS:
         break;
 
     case PROBLEM:
         if (*pzOopsPrefix != NUL) {
             fprintf( stdout, "%soutput was abandoned\n", pzOopsPrefix );
-            pzOopsPrefix = "";
+            pzOopsPrefix = zNil;
         }
         fclose( stdout );
         return;
 
     default:
         if (*pzOopsPrefix != NUL) {
-            fprintf( stdout, zBadR, pzOopsPrefix, jmpcode );
-            pzOopsPrefix = "";
+            fprintf( stdout, zBadR, pzOopsPrefix );
+            pzOopsPrefix = zNil;
         } else {
-            fprintf( pfTrace, zBadR+2, jmpcode );
+            fprintf( pfTrace, zBadR+2 );
         }
 
     case FAILURE:
@@ -153,10 +156,10 @@ doStdoutTemplate( tTemplate* pTF )
         res   = ag_scm_out_pop( SCM_BOOL_T );
         pzRes = AG_SCM_CHARS( res );
 
-        if (strneqvcmp( pzRes, zCont, sizeof( zCont ) - 1) != 0)
+        if (strneqvcmp( pzRes, zCont, (int)sizeof(zCont) - 1) != 0)
             fputs( "Content-Type: text/html\n\n", stdout );
 
-        fwrite( pzRes, AG_SCM_STRLEN( res ), 1, stdout );
+        fwrite( pzRes, AG_SCM_STRLEN(res), (size_t)1, stdout );
     }
 
     fclose( stdout );
@@ -180,7 +183,6 @@ processTemplate( tTemplate* pTF )
 
     for (;;) {
         tOutSpec*  pOS    = pOutSpecList;
-        int        jumpCd = setjmp( fileAbort );
 
         /*
          * We cannot be in Scheme processing.  We've either just started
@@ -193,7 +195,8 @@ processTemplate( tTemplate* pTF )
         /*
          *  HOW was that we got here?
          */
-        if (jumpCd == SUCCESS) {
+        switch (setjmp( fileAbort )) {
+        case SUCCESS:
             /*
              *  Set the output file name buffer.
              *  It may get switched inside openOutFile.
@@ -211,15 +214,23 @@ processTemplate( tTemplate* pTF )
             do  {
                 closeOutput( AG_FALSE );  /* keep output */
             } while (pCurFp->pPrev != NULL);
-        }
+            break;
 
-        else {
-            if ((jumpCd != PROBLEM) && (jumpCd != FAILURE)) {
-                fprintf( pfTrace, "%sBogus return from setjmp:  %d\n",
-                         pzOopsPrefix, jumpCd );
-                pzOopsPrefix = "";
-            }
+        case PROBLEM:
+            /*
+             *  We got here by a long jump.  Close/purge the open files.
+             */
+            do  {
+                closeOutput( AG_TRUE );  /* discard output */
+            } while (pCurFp->pPrev != NULL);
+            break;
 
+        default:
+            fprintf( pfTrace, "%sBogus return from setjmp\n", pzOopsPrefix );
+            pzOopsPrefix = zNil;
+            /* FALLTHROUGH */
+
+        case FAILURE:
             /*
              *  We got here by a long jump.  Close/purge the open files.
              */
@@ -230,10 +241,8 @@ processTemplate( tTemplate* pTF )
             /*
              *  On failure (or unknown jump type), we quit the program, too.
              */
-            if (jumpCd != PROBLEM) {
-                procState = PROC_STATE_ABORTING;
-                exit( EXIT_FAILURE );
-            }
+            procState = PROC_STATE_ABORTING;
+            exit( EXIT_FAILURE );
         }
 
         /*
@@ -311,15 +320,19 @@ openOutFile( tOutSpec* pOutSpec, tFpStack* pStk )
      *  Remove any suffixes in the last file name
      */
     {
-        char   z[ MAXPATHLEN ];
+        char   z[ AG_PATH_MAX ];
         tCC*   pS = strrchr( pzDefFile, '/' );
         char*  pE;
-        if (pS == NULL)
+        if (pS++ == NULL)
             pS = pzDefFile;
 
         pE = strchr( pS, '.' );
         if (pE != NULL) {
-            memcpy( z, pS, pE - pS );
+            size_t len = (unsigned)(pE - pS);
+            if (len >= sizeof(z))
+                AG_ABEND( "--base-name name is too long" );
+
+            memcpy(z, pS, len);
             z[ pE - pS ] = NUL;
             pS = z;
         }
@@ -380,7 +393,6 @@ openOutFile( tOutSpec* pOutSpec, tFpStack* pStk )
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of agen5/tpProcess.c */
