@@ -1,8 +1,8 @@
 
 /*
- *  Time-stamp:      "2005-09-20 19:32:35 bkorb"
+ *  Time-stamp:      "2006-12-02 08:45:02 bkorb"
  *
- *  autoopts.h  $Id: autoopts.h,v 4.15 2005/10/02 16:34:30 bkorb Exp $
+ *  autoopts.h  $Id: autoopts.h,v 4.19 2006/12/10 16:43:01 bkorb Exp $
  *  Time-stamp:      "2005-02-14 05:59:50 bkorb"
  *
  *  This file defines all the global structures and special values
@@ -10,7 +10,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2005 Bruce Korb
+ *  Automated Options copyright 1992-2006 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -58,24 +58,30 @@
 #include "compat/compat.h"
 
 #define AO_NAME_LIMIT    127
-#define AO_NAME_SIZE     (AO_NAME_LIMIT + 1)
+#define AO_NAME_SIZE     ((size_t)(AO_NAME_LIMIT + 1))
 
-#ifndef MAXPATHLEN
+#ifndef AG_PATH_MAX
 #  ifdef PATH_MAX
-#    define MAXPATHLEN   PATH_MAX
+#    define AG_PATH_MAX   ((size_t)PATH_MAX)
 #  else
-#    define MAXPATHLEN   4096
+#    define AG_PATH_MAX   ((size_t)4096)
 #  endif
 #else
 #  if defined(PATH_MAX) && (PATH_MAX > MAXPATHLEN)
-#     undef  MAXPATHLEN
-#     define MAXPATHLEN  PATH_MAX
+#     undef  AG_PATH_MAX
+#     define AG_PATH_MAX  ((size_t)PATH_MAX)
 #  endif
 #endif
 
 #undef  EXPORT
 #define EXPORT
 
+#if defined(_WIN32) && !defined(__CYGWIN__)
+# define DIRCH '\\'
+#else
+# define DIRCH '/'
+#endif
+
 /*
  *  Convert the number to a list usable in a printf call
  */
@@ -101,7 +107,18 @@ typedef int tDirection;
  *
  *  USAGE:  define procedures to return "tSuccess".  Test their results
  *          with the SUCCEEDED, FAILED and HADGLITCH macros.
+ *
+ *  Microsoft sticks its nose into user space here, so for Windows' sake,
+ *  make sure all of these are undefined.
  */
+#undef  SUCCESS
+#undef  FAILURE
+#undef  PROBLEM
+#undef  SUCCEEDED
+#undef  SUCCESSFUL
+#undef  FAILED
+#undef  HADGLITCH
+
 #define SUCCESS  ((tSuccess) 0)
 #define FAILURE  ((tSuccess)-1)
 #define PROBLEM  ((tSuccess) 1)
@@ -145,7 +162,7 @@ typedef enum {
 typedef struct {
     tOptDesc*  pOD;
     tCC*       pzOptArg;
-    tUL        flags;
+    tAoUL      flags;
     teOptType  optType;
 } tOptState;
 #define OPTSTATE_INITIALIZER(st) \
@@ -178,15 +195,24 @@ typedef struct {
     tCC*    pzOptFmt;
 } arg_types_t;
 
-#  define AGALOC( c, w )        malloc( c )
-#  define AGREALOC( p, c, w )   realloc( p, c )
-#  define AGFREE( p )           free( p )
-#  define AGDUPSTR( p, s, w )   p = strdup( s )
-#  define TAGMEM( m, t )
+#define AGALOC( c, w )        ao_malloc((size_t)c)
+#define AGREALOC( p, c, w )   ao_realloc(p, (size_t)c)
+#define AGFREE( p )           ao_free(p)
+#define AGDUPSTR( p, s, w )   (p = ao_strdup(s))
 
-#ifdef AUTOGEN_BUILD
-#  include <snprintfv/printf.h>
-#endif /* AUTOGEN_BUILD */
+static void *
+ao_malloc( size_t sz );
+
+static void *
+ao_realloc( void *p, size_t sz );
+
+static void
+ao_free( void *p );
+
+static char *
+ao_strdup( char const *str );
+
+#define TAGMEM( m, t )
 
 /*
  *  DO option handling?
@@ -292,6 +318,11 @@ typedef struct {
 # endif
 #endif
 
+#ifndef HAVE_STRCHR
+extern char* strchr( char const *s, int c);
+extern char* strrchr( char const *s, int c);
+#endif
+
 /*
  *  Define and initialize all the user visible strings.
  *  We do not do translations.  If translations are to be done, then
@@ -307,15 +338,11 @@ extern FILE* option_usage_fp;
 
 extern tOptProc optionPrintVersion, optionPagedUsage, optionLoadOpt;
 
-#define LOCAL static
-#include "proto.h"
-
 #endif /* AUTOGEN_AUTOOPTS_H */
 /*
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of autoopts/autoopts.h */
