@@ -1,7 +1,7 @@
 
 /*
- *  $Id: enumeration.c,v 4.8 2005/12/09 20:13:50 bkorb Exp $
- * Time-stamp:      "2005-12-09 06:37:15 bkorb"
+ *  $Id: enumeration.c,v 4.14 2006/11/27 01:52:23 bkorb Exp $
+ * Time-stamp:      "2006-11-26 14:32:33 bkorb"
  *
  *   Automated Options Paged Usage module.
  *
@@ -10,7 +10,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2005 Bruce Korb
+ *  Automated Options copyright 1992-2006 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -58,33 +58,33 @@ tSCC*  pz_enum_err_fmt;
 /* static forward declarations maintained by :mkfwd */
 static void
 enumError(
-    tOptions* pOpts,
-    tOptDesc* pOD,
-    tCC**     paz_names,
-    int       name_ct );
+    tOptions*     pOpts,
+    tOptDesc*     pOD,
+    tCC* const *  paz_names,
+    int           name_ct );
 
 static uintptr_t
 findName(
     tCC*          pzName,
     tOptions*     pOpts,
     tOptDesc*     pOD,
-    tCC**         paz_names,
+    tCC* const *  paz_names,
     unsigned int  name_ct );
 /* = = = END-STATIC-FORWARD = = = */
 
 static void
 enumError(
-    tOptions* pOpts,
-    tOptDesc* pOD,
-    tCC**     paz_names,
-    int       name_ct )
+    tOptions*     pOpts,
+    tOptDesc*     pOD,
+    tCC* const *  paz_names,
+    int           name_ct )
 {
     size_t max_len = 0;
     size_t ttl_len = 0;
 
     if (pOpts != NULL)
-        fprintf( option_usage_fp, pz_enum_err_fmt,
-                 pOpts->pzProgName, pOD->pzLastArg );
+        fprintf( option_usage_fp, pz_enum_err_fmt, pOpts->pzProgName,
+                 pOD->optArg.argString, pOD->pz_Name );
 
     fprintf( option_usage_fp, zValidKeys, pOD->pz_Name );
 
@@ -98,7 +98,7 @@ enumError(
      *  of all the names.
      */
     {
-        tCC** paz = paz_names;
+        tCC * const * paz = paz_names;
         int   ct  = name_ct;
 
         do  {
@@ -137,7 +137,7 @@ enumError(
         int   ent_no = 0;
         char  zFmt[16];  /* format for all-but-last entries on a line */
 
-        sprintf( zFmt, "%%-%ds", max_len );
+        sprintf( zFmt, "%%-%ds", (int)max_len );
         max_len = 78 / max_len; /* max_len is now max entries on a line */
         fputs( "  ", option_usage_fp );
 
@@ -175,7 +175,7 @@ findName(
     tCC*          pzName,
     tOptions*     pOpts,
     tOptDesc*     pOD,
-    tCC**         paz_names,
+    tCC* const *  paz_names,
     unsigned int  name_ct )
 {
     uintptr_t     res = name_ct;
@@ -186,14 +186,14 @@ findName(
      *  Multiple partial matches means we have an ambiguous match.
      */
     for (idx = 0; idx < name_ct; idx++) {
-        if (strncmp( (char*)paz_names[idx], (char*)pzName, len ) == 0) {
+        if (strncmp( (char*)paz_names[idx], (char*)pzName, len) == 0) {
             if (paz_names[idx][len] == NUL)
                 return idx;  /* full match */
 
             if (res != name_ct) {
                 pz_enum_err_fmt = zAmbigKey;
                 option_usage_fp = stderr;
-                enumError( pOpts, pOD, paz_names, name_ct );
+                enumError( pOpts, pOD, paz_names, (int)name_ct );
             }
             res = idx; /* save partial match */
         }
@@ -205,7 +205,7 @@ findName(
     if (res == name_ct) {
         pz_enum_err_fmt = zNoKey;
         option_usage_fp = stderr;
-        enumError( pOpts, pOD, paz_names, name_ct );
+        enumError( pOpts, pOD, paz_names, (int)name_ct );
     }
 
     /*
@@ -223,21 +223,21 @@ findName(
  * arg:   tOptDesc*,     pOD,       enumeration option description
  * arg:   unsigned int,  enum_val,  the enumeration value to map
  *
- * ret_type:  const char*
+ * ret_type:  char const*
  * ret_desc:  the enumeration name from const memory
  *
  * doc:   This converts an enumeration value into the matching string.
 =*/
-const char*
+char const*
 optionKeywordName(
     tOptDesc*     pOD,
     unsigned int  enum_val )
 {
     tOptDesc od;
 
-    od.pzLastArg = (const char*)(uintptr_t)enum_val;
+    od.optArg.argEnum = enum_val;
     (*(pOD->pOptProc))( (void*)(2UL), &od );
-    return od.pzLastArg;
+    return od.optArg.argString;
 }
 
 
@@ -247,23 +247,23 @@ optionKeywordName(
  *
  * arg:   tOptions*,     pOpts,     the program options descriptor
  * arg:   tOptDesc*,     pOD,       enumeration option description
- * arg:   const char**,  paz_names, list of enumeration names
+ * arg:   char const * const *,  paz_names, list of enumeration names
  * arg:   unsigned int,  name_ct,   number of names in list
  *
- * ret_type:  char*
- * ret_desc:  the enumeration value cast as a char*
+ * ret_type:  uintptr_t
+ * ret_desc:  the enumeration value
  *
- * doc:   This converts the pzLastArg string from the option description
+ * doc:   This converts the optArg.argString string from the option description
  *        into the index corresponding to an entry in the name list.
  *        This will match the generated enumeration value.
  *        Full matches are always accepted.  Partial matches are accepted
  *        if there is only one partial match.
 =*/
-char*
+uintptr_t
 optionEnumerationVal(
     tOptions*     pOpts,
     tOptDesc*     pOD,
-    tCC**         paz_names,
+    tCC * const * paz_names,
     unsigned int  name_ct )
 {
     /*
@@ -275,12 +275,12 @@ optionEnumerationVal(
         /*
          *  print the list of enumeration names.
          */
-        enumError( pOpts, pOD, paz_names, name_ct );
-        return (char*)0UL;
+        enumError( pOpts, pOD, paz_names, (int)name_ct );
+        return 0UL;
 
     case 1UL:
     {
-        unsigned int ix = (uintptr_t)(pOD->pzLastArg);
+        unsigned int ix = pOD->optArg.argEnum;
         /*
          *  print the name string.
          */
@@ -288,25 +288,25 @@ optionEnumerationVal(
             printf( "INVALID-%d", ix );
         else
             fputs( paz_names[ ix ], stdout );
-        return (char*)0UL;
+        return 0UL;
     }
     case 2UL:
     {
         tSCC zInval[] = "*INVALID*";
-        unsigned int ix = (uintptr_t)(pOD->pzLastArg);
+        unsigned int ix = pOD->optArg.argEnum;
         /*
          *  Replace the enumeration value with the name string.
          */
         if (ix >= name_ct)
-            return (char*)zInval;
+            return (uintptr_t)zInval;
 
-        return (char*)paz_names[ ix ];
+        return (uintptr_t)paz_names[ ix ];
     }
     default:
         break;
     }
 
-    return (char*)findName( pOD->pzLastArg, pOpts, pOD, paz_names, name_ct );
+    return findName( pOD->optArg.argString, pOpts, pOD, paz_names, name_ct );
 }
 
 
@@ -316,10 +316,11 @@ optionEnumerationVal(
  *
  * arg:   tOptions*,     pOpts,     the program options descriptor
  * arg:   tOptDesc*,     pOD,       enumeration option description
- * arg:   const char**,  paz_names, list of enumeration names
+ * arg:   char const * const *,
+ *                       paz_names, list of enumeration names
  * arg:   unsigned int,  name_ct,   number of names in list
  *
- * doc:   This converts the pzLastArg string from the option description
+ * doc:   This converts the optArg.argString string from the option description
  *        into the index corresponding to an entry in the name list.
  *        This will match the generated enumeration value.
  *        Full matches are always accepted.  Partial matches are accepted
@@ -329,7 +330,7 @@ void
 optionSetMembers(
     tOptions*     pOpts,
     tOptDesc*     pOD,
-    tCC**         paz_names,
+    tCC* const *  paz_names,
     unsigned int  name_ct )
 {
     /*
@@ -341,7 +342,7 @@ optionSetMembers(
         /*
          *  print the list of enumeration names.
          */
-        enumError( pOpts, pOD, paz_names, name_ct );
+        enumError( pOpts, pOD, paz_names, (int)name_ct );
         return;
 
     case 1UL:
@@ -382,7 +383,7 @@ optionSetMembers(
             bits >>= 1;
         }
 
-        pOD->pzLastArg = pz = malloc( len );
+        pOD->optArg.argString = pz = malloc( len );
         /*
          *  Start by clearing all the bits.  We want to turn off any defaults
          *  because we will be restoring to current state, not adding to
@@ -409,7 +410,7 @@ optionSetMembers(
     }
 
     {
-        tCC*      pzArg = pOD->pzLastArg;
+        tCC*      pzArg = pOD->optArg.argString;
         uintptr_t res;
         if ((pzArg == NULL) || (*pzArg == NUL)) {
             pOD->optCookie = (void*)0;
@@ -430,12 +431,12 @@ optionSetMembers(
             if (len == 0)
                 break;
 
-            if ((len == 3) && (strncmp( pzArg, zAll, 3 ) == 0)) {
+            if ((len == 3) && (strncmp(pzArg, zAll, (size_t)3) == 0)) {
                 if (iv)
                      res = 0;
-                else res = ~0;
+                else res = ~0UL;
             }
-            else if ((len == 4) && (strncmp( pzArg, zNone, 4 ) == 0)) {
+            else if ((len == 4) && (strncmp(pzArg, zNone, (size_t)4) == 0)) {
                 if (! iv)
                     res = 0;
             }
@@ -449,7 +450,7 @@ optionSetMembers(
                     if (*pz != NUL) {
                         if (len >= AO_NAME_LIMIT)
                             break;
-                        strncpy( z, pzArg, len );
+                        strncpy( z, pzArg, (size_t)len );
                         z[len] = NUL;
                         p = z;
                     } else {
@@ -479,7 +480,6 @@ optionSetMembers(
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of autoopts/enumeration.c */
