@@ -1,7 +1,7 @@
 
 /*
- *  $Id: load.c,v 4.17 2005/11/07 00:22:05 bkorb Exp $
- *  Time-stamp:      "2005-10-29 14:45:36 bkorb"
+ *  $Id: load.c,v 4.15 2006/11/27 01:52:23 bkorb Exp $
+ *  Time-stamp:      "2006-11-26 14:33:07 bkorb"
  *
  *  This file contains the routines that deal with processing text strings
  *  for options, either from a NUL-terminated string passed in or from an
@@ -9,7 +9,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2005 Bruce Korb
+ *  Automated Options copyright 1992-2006 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -77,8 +77,8 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode );
  * what:  translate and construct a path
  * arg:   + char*       + pzBuf      + The result buffer +
  * arg:   + int         + bufSize    + The size of this buffer +
- * arg:   + const char* + pzName     + The input name +
- * arg:   + const char* + pzProgPath + The full path of the current program +
+ * arg:   + char const* + pzName     + The input name +
+ * arg:   + char const* + pzProgPath + The full path of the current program +
  *
  * ret-type: ag_bool
  * ret-desc: AG_TRUE if the name was handled, otherwise AG_FALSE.
@@ -102,7 +102,7 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode );
  *
  *  Please note: both @code{$$} and @code{$NAME} must be at the start of the
  *     @code{pzName} string and must either be the entire string or be followed
- *     by the @code{'/'} character.
+ *     by the @code{'/'} (backslash on windows) character.
  *
  * err:  @code{AG_FALSE} is returned if:
  *       @*
@@ -195,7 +195,7 @@ insertProgramPath(
     int     skip = 2;
 
     switch (pzName[2]) {
-    case '/':
+    case DIRCH:
         skip = 3;
     case NUL:
         break;
@@ -208,7 +208,7 @@ insertProgramPath(
      *  If it is, we're done.  Otherwise, we have to hunt
      *  for the program using "pathfind".
      */
-    if (strchr( pzProgPath, '/' ) != NULL)
+    if (strchr( pzProgPath, DIRCH ) != NULL)
         pzPath = pzProgPath;
     else {
         pzPath = pathfind( getenv( "PATH" ), (char*)pzProgPath, "rx" );
@@ -217,7 +217,7 @@ insertProgramPath(
             return AG_FALSE;
     }
 
-    pz = strrchr( pzPath, '/' );
+    pz = strrchr( pzPath, DIRCH );
 
     /*
      *  IF we cannot find a directory name separator,
@@ -235,7 +235,7 @@ insertProgramPath(
     if ((pz - pzPath)+1 + strlen(pzName) >= bufSize)
         return AG_FALSE;
 
-    memcpy( pzBuf, pzPath, (pz - pzPath)+1 );
+    memcpy( pzBuf, pzPath, (size_t)((pz - pzPath)+1) );
     strcpy( pzBuf + (pz - pzPath) + 1, pzName );
 
     /*
@@ -258,10 +258,10 @@ insertEnvVal(
     char* pzDir = pzBuf;
 
     for (;;) {
-        char ch = *++pzName;
+        int ch = (int)*++pzName;
         if (! ISNAMECHAR( ch ))
             break;
-        *(pzDir++) = ch;
+        *(pzDir++) = (char)ch;
     }
 
     if (pzDir == pzBuf)
@@ -293,16 +293,16 @@ mungeString( char* pzTxt, tOptionLoadMode mode )
     if (mode == OPTION_LOAD_KEEP)
         return;
 
-    if (isspace( *pzTxt )) {
+    if (isspace( (int)*pzTxt )) {
         char* pzS = pzTxt;
         char* pzD = pzTxt;
-        while (isspace( *++pzS ))  ;
+        while (isspace( (int)*++pzS ))  ;
         while ((*(pzD++) = *(pzS++)) != NUL)   ;
         pzE = pzD-1;
     } else
         pzE = pzTxt + strlen( pzTxt );
 
-    while ((pzE > pzTxt) && isspace( pzE[-1] ))  pzE--;
+    while ((pzE > pzTxt) && isspace( (int)pzE[-1] ))  pzE--;
     *pzE = NUL;
 
     if (mode == OPTION_LOAD_UNCOOKED)
@@ -352,9 +352,9 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode )
      *  because we'll have to skip over an immediately following ':' or '='
      *  (and the white space following *that*).
      */
-    space_break = isspace(*pzEnd);
+    space_break = isspace((int)*pzEnd);
     *(pzEnd++) = NUL;
-    while (isspace(*pzEnd))  pzEnd++;
+    while (isspace((int)*pzEnd))  pzEnd++;
     if (space_break && ((*pzEnd == ':') || (*pzEnd == '=')))
         pzEnd++;
 
@@ -377,7 +377,7 @@ loadOptionLine(
     tDirection  direction,
     tOptionLoadMode   load_mode )
 {
-    while (isspace( *pzLine ))  pzLine++;
+    while (isspace( (int)*pzLine ))  pzLine++;
 
     {
         char* pzArg = assembleArgValue( pzLine, load_mode );
@@ -479,7 +479,7 @@ loadOptionLine(
  * what:  process a string for an option name and value
  *
  * arg:   tOptions*,   pOpts,  program options descriptor
- * arg:   const char*, pzLine, NUL-terminated text
+ * arg:   char const*, pzLine, NUL-terminated text
  *
  * doc:
  *
@@ -514,7 +514,6 @@ optionLoadLine(
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of autoopts/load.c */
